# -*- coding: utf-8 -*-

from django.contrib import admin
from django.core.urlresolvers import reverse
from django.utils.translation import ugettext_lazy as _
from django.utils.translation import ugettext as ugt
from django.conf import settings
from django.db import transaction
from django import forms
from django.contrib.admin import widgets                                       

from jmb.core.admin.options import ExtendibleModelAdmin
from jmb.core.forms.models import AdminModelForm
from jmb.core.forms.fields import JumboDateField
from jmb.core.db.models import STATUS_CODES

from jmb.openvpn.models import Key, Certificate, Connection, FwRule, FwGroup
from jmb.openvpn import views as openvpn_views
import django_filters

# class CertSearchFilterset(django_filters.FilterSet):
# #    start_date = django_filters.DateFilter(required=False, lookup_type='gte')
#     start_date = django_filters.CustomDateRangeFilter(lookup_type='gte')
# #    status = django_filters.MultipleChoiceFilter()
#     class Meta:
#         model = Certificate
#         display_fields = (
#             ('start_date',),
#             ('user__username', 'user__last_name', ),
#             ('status', 'description',),
#             )
#         fields = display_fields
# #        fields = [item for sublist in display_fields for item in sublist]

#     def __init__(self, *args, **kwargs):
#         django_filters.FilterSet.__init__(self, *args, **kwargs)
#         self.filters['start_date'].lookup_type = 'gte'
#         self.filters['user__username'].lookup_type = ('icontains','startswith')
# #        self.filters['status'].field.widget.choices.insert(0,('', ''))

# class CertSearchForm(forms.Form):
#     class Meta:
#         model = Certificate
#         fields = []
#         display_fields = (
#             ('start_date__gte',),
#             ('user__username__icontains', 'user__last_name__icontains', ),
#             ('status', 'description__icontains',),
#             )

#     def __init__(self, *args, **kws):
#         super(CertSearchForm, self).__init__(*args, **kws)
#         codes = list(STATUS_CODES)
#         codes.insert(0, ('', '-------'))
#         self.fields['description__icontains'] = forms.CharField(
#             label=_("Description"), required=False)
#         self.fields['user__username__icontains'] = forms.CharField(
#             label=_("Utente"), required=False)
#         self.fields['user__last_name__icontains'] = forms.CharField(
#             label=_("cognome"), required=False)
#         self.fields['status'] = forms.ChoiceField(
#             choices=codes, label=_("Status"), required=False)
#         self.fields['start_date__gte'] = forms.DateField(
#             required=False, widget=widgets.AdminDateWidget())


class KeyAdmin(ExtendibleModelAdmin):
    model = Key
    list_display = (
        'get_status_icon', 'user', 'identity', 'get_edit_icon',
        'get_detail_icon', 'get_delete_icon'
    )
    fieldsets = (
        (_('logs admin'), {
            'fields': (
                'creator', 'date_create', 'last_modifier', 'date_last_modify',
            ),
            'classes': ('collapse',)
        }),
        (None, {"fields": (
            'status', 'user', 'identity', 'path_pub', 'path_priv',
        ),}),
    )

class CertificateForm(AdminModelForm):
    def __init__(self, *args, **kwargs):
        super(CertificateForm, self).__init__(*args, **kwargs)
        self.fields['user'].required= True

class CertificateAdmin(ExtendibleModelAdmin):
    model = Certificate
    form = CertificateForm
    list_display = (
        'get_status_icon', 'user', 'description', 'start_date', 'stop_date', 'cn',
        'get_edit_icon', 'get_download_icon', 'get_delete_icon',
#        'get_edit_icon', 'get_detail_icon', 'get_download_icon', 'get_delete_icon', FIXME
    )
    default_group_by = 'group'
#    search_fields = ['cn','description']
#    search_filterset_class = CertSearchFilterset
    advanced_search_fields = (
            ('start_date__gte', 'user',),
            ('user__username__icontains', 'user__last_name__icontains', ),
            ('status', 'description__icontains',),
            )
#    search_form_class = CertSearchForm
    date_hierarchy = 'stop_date'
    list_filter = ['stop_date','user']
    fieldsets = (
        (_('logs admin'), {
            'fields': (
                ('creator', 'date_create'), ('last_modifier', 'date_last_modify'),
            ),
            'classes': ('collapse',)
        }),
        (None, {"fields": (
            'status', ('user', 'group',), ('start_date', 'stop_date',),
            'description',
        ),}),
    )

    def get_readonly_fields(self, request, obj = None):
        # FIXME : Per ora rendo non editabili le date in modifica.
        # Sarebbe piu carino fare in modo che se le date cambiano il certificato si aggiorna.
        # Devo chiedere a Sandro
        if obj: #In edit mode
            return ('start_date', 'stop_date') + self.readonly_fields
        return self.readonly_fields

    def get_urls(self):
        from django.conf.urls.defaults import patterns, url
        urls = super(CertificateAdmin, self).get_urls()
        my_urls = patterns('',
            url(
                r'^certificate/download/(?P<object_id>\d+)/$',
                self.admin_site.admin_view(openvpn_views.download_certificate),
                name = 'openvpn-certificate-download'
            )
        )
        return my_urls + urls

    def get_download_icon(self, result):
        return """
            <a href=%(reverse_url)s>
                <img src='%(url)sjmb/images/document.png' alt='%(window_title)s' />
            </a>
        """ % {
            'reverse_url':reverse('admin:openvpn-certificate-download', args=(result.id,)),
            'url':settings.STATIC_URL,
            'window_title':ugt("Download Certificate")
        }
    get_download_icon.short_description = _("C")
    get_download_icon.allow_tags = True

class ConnectionAdmin(ExtendibleModelAdmin):
    model = Connection
    list_display = (
        'certificate', 'start_date', 'stop_date',
        'ip_client', 'get_edit_icon', 'get_detail_icon', 'get_delete_icon'
    )
    search_fields = ['certificate__description']
    date_hierarchy = 'stop_date'
    list_filter = ['stop_date']
    
    fieldsets = (
        (_('logs admin'), {
            'fields': ('date_create', 'date_last_modify',),
            'classes': ('collapse',)
        }),
        (None, {"fields": (
            'certificate', 'stop_date', 'ip_client'
        ),}),
    )

    def get_urls(self):
        from django.conf.urls.defaults import patterns, url
        urls = super(ConnectionAdmin, self).get_urls()
        my_urls = patterns('',
            url(
                r'^get_status_openvpn/$',
                self.admin_site.admin_view(openvpn_views.get_status_openvpn),
                name='openvpn_openvpn_get_status'
            ),
            url(
                r'^on_off_openvpn/$',
                self.admin_site.admin_view(openvpn_views.onoff_openvpn),
                name='openvpn_openvpn_onoff'
            ),
            url(
                r'^restart_openvpn/$',
                self.admin_site.admin_view(openvpn_views.restart_openvpn),
                name='openvpn_openvpn_restart'
            ),
        )
        return my_urls + urls

class FwRuleAdmin(ExtendibleModelAdmin):
    model = FwRule
    list_display = (
        'get_status_icon', 'get_groups', 'ip_src', 'ip_dest',
        'proto', 'port_dest', 'action', 'order',
        'get_edit_icon', 'get_detail_icon', 'get_delete_icon'
    )
    default_group_by = 'chain'
    fieldsets = (
        (_('logs admin'), {
            'fields': (
                'creator', 'date_create', 'last_modifier', 'date_last_modify',
            ),
            'classes': ('collapse',)
        }),
        (None, {"fields": (
            'status', 'groups',
            ('table', 'chain',),
            ('ip_src', 'ip_dest',),
            ('proto', 'port_dest',),
            ('action',  'order'),
        ),}),
    )

class FwGroupAdmin(ExtendibleModelAdmin):
    model = FwGroup
    list_display = (
        'get_status_icon', 'name',
        'get_edit_icon', 'get_detail_icon', 'get_delete_icon'
    )
    fieldsets = (
        (_('logs admin'), {
            'fields': (
                'creator', 'date_create', 'last_modifier', 'date_last_modify',
            ),
            'classes': ('collapse',)
        }),
        (None, {"fields": (
            'status', 'name',
        ),}),
    )

admin.site.register(Key, KeyAdmin)
admin.site.register(Certificate, CertificateAdmin)
admin.site.register(Connection, ConnectionAdmin)
admin.site.register(FwRule, FwRuleAdmin)
admin.site.register(FwGroup, FwGroupAdmin)
