import datetime

from django import forms
from django.utils.translation import ugettext_lazy as _
from django.forms.util import ValidationError, ErrorList
from django.utils.encoding import force_unicode
from django.contrib.auth.models import User

from jmb.core.forms import ModelForm, ModelSearchForm, DateWidget, DateTimeWidget, MultiFileField
from jmb.core.db.forms import AdminModelForm
from jmb.core.middleware.thread_local import get_request

from jmb.openvpn.models import *

class UtentiSearchForm(ModelSearchForm):
    class Meta:
        model = Utenti
        fields = ('username',)

class CertificateSearchForm(ModelSearchForm):
    class Meta:
        model = Certificate
        fields = ('user',)

class FwruleSearchForm(ModelSearchForm):
    class Meta:
        model = FwRule
        fields = ('ip_src',)

class FwRuleForm(ModelForm):
    class Meta:
        model = FwRule

    def save(self):
        rule = super(FwRuleForm, self).save()

        # Cerco tt i certificati che hanno come gruppo uno di quelli
        # associati alla regola che si sta salvando, e ne chiamo il metodo
        # refresh_rules che crea/modifica le relative regole.
        #
        for group in rule.group.all():
            l_cert = Certificate.objects.filter(group__exact=group.id)
            for c in l_cert:
                c.refresh_rules()
        return rule

class FwgroupSearchForm(ModelSearchForm):
    class Meta:
        model = FwGroup
        fields = ('name',)

class ConnectionSearchForm(ModelSearchForm):
    class Meta:
        model = Connection
        fields = ('certificate',)

    def __init__(self, *args, **kwargs):
        super(ConnectionSearchForm, self).__init__(*args, **kwargs)
        self.fields['certificate__description'] = forms.CharField(label=_("Description"), required=False)
