from django.utils.translation import ugettext as _
from django.core.urlresolvers import reverse
from admin_tools.menu import items, Menu

from jmb.core.conf.admin_menu import GenericMenuItem
from jmb.core.middleware.thread_local import get_current_user

def get_root_menu(children):
    user = get_current_user()
    if user.has_perm("jmb.openvpn.manage_openvpn"):
        root_menu = items.MenuItem(
            title=_('OpenVPN'),
            children=[],
            url = reverse('admin:jmb.openvpn_certificate_changelist')
        )
        children.append(root_menu)
    return None

def add_extended_menuitems(children, current_app):
    generic_menu = GenericMenuItem(current_app, ['certificate', 'connection', 'fwrule', 'fwgroup'])
    user = get_current_user()

    get_status_openvpn = items.MenuItem(
        title=_('Get Status VPN'),
        children=[],
        url = reverse('admin:jmb.openvpn_openvpn_get_status')
    )
    if user.has_perm('jmb.openvpn.get_status_openvpn'):
        generic_menu.add_child(get_status_openvpn, 'connection')

    onoff_openvpn = items.MenuItem(
        title=_('On / Off VPN'),
        children=[],
        url = reverse('admin:jmb.openvpn_openvpn_onoff')
    )
    if user.has_perm('jmb.openvpn.onoff_openvpn'):
        generic_menu.add_child(onoff_openvpn, 'connection')

    restart_openvpn = items.MenuItem(
        title=_('Restart VPN'),
        children=[],
        url = reverse('admin:jmb.openvpn_openvpn_restart')
    )
    if user.has_perm('jmb.openvpn.restart'):
        generic_menu.add_child(restart_openvpn, 'connection')

    menus = generic_menu.get_menu()
    [ children.append(menu) for menu in menus ]
