import sys
import datetime
from optparse import make_option

from django.core.management.base import BaseCommand, CommandError
from jmb.openvpn.models import Certificate, Connection

class Command(BaseCommand):
    option_list = BaseCommand.option_list + (
        make_option('-c', '--common_name', dest='common-name',
            help="The Certificate's common name."),
    )
    help = "Save a close connection for the user."

    # Validation is called explicitly each time the server is reloaded.
    requires_model_validation = False

    def handle(self, *args, **options):

        cn = options.get('common-name',None)
        TODAY = datetime.date.today()


        if cn:
            certs = Certificate.objects.filter(start_date__lte=TODAY, cn=cn,
                                           stop_date__gte=TODAY, status=1)
            if not certs:
                sys.exit(1)
            cert = certs[0]
            # Cerco tutte le connessioni fatte dal common_name,
            # prendo quella che non ha la data di stop e la inserisco
            #
            conn = Connection.objects.filter(certificate__id=cert.id, stop_date__isnull=True)

            for c in conn:
                c.stop_date = datetime.datetime.now()
                c.save()
