import sys
import datetime
from optparse import make_option

from django.core.management.base import BaseCommand, CommandError
from django.conf import settings
from jmb.openvpn.models import Certificate


class Command(BaseCommand):
    option_list = BaseCommand.option_list + (
        make_option('-u', '--username', dest='username',
            help='Specifies the username.'),
        make_option('-p', '--password', dest='password',
            help='Specifies the password.'),
    )
    help = "Check the user credentials."

    # Validation is called explicitly each time the server is reloaded.
    requires_model_validation = False

    def handle(self, *args, **options):

        username = options.get('username', None)
        pwd = options.get('password', None)
        TODAY = datetime.date.today()

        if not username or not pwd:
            print "Tutti i campi sono obbligatori"
            sys.exit(1)

        # Controllo la password inserita
        try:
            certs = Certificate.objects.filter(start_date__lte=TODAY, cn=username,
                                           stop_date__gte=TODAY, status=1)
            if not certs:
                print "Missing certificate for username: '%s'" % username
                sys.exit(1)
            cert = certs[0]

            if not cert.user.check_password(pwd) and settings.OVPN_USE_FALLBACK_PASSWD:
                if cert.user.check_password(settings.OVPN_USE_FALLBACK_PASSWD):
                    return
                print "Password Errata"
                sys.exit(1)
        except Exception, e:
            print "Certificato non trovato (%s)" % e
            sys.exit(1)
        print "ok"
