import sys
import datetime
from optparse import make_option

from django.core.management.base import BaseCommand, CommandError

from jmb.openvpn.models import Certificate, Connection

# FIXME: richiede che esista solo 1 certificato

class Command(BaseCommand):
    option_list = BaseCommand.option_list + (
        make_option('-c', '--common_name', dest='common-name',
            help="The Certificate's common name."),
        make_option('-i', '--ip_client', dest='ip-client',
            help='Specifies the IP Client.'),
    )
    help = "Save an open connection for the user."

    # Validation is called explicitly each time the server is reloaded.
    requires_model_validation = False

    def handle(self, *args, **options):

        TODAY = datetime.date.today()

        cn = options.get('common-name',None)
        ip_client = options.get('ip-client', '0.0.0.0')

        if cn:
            certs = Certificate.objects.filter(start_date__lte=TODAY, cn=cn,
                                           stop_date__gte=TODAY, status=1)
            if not certs:
                sys.exit(1)
            c = certs[0]
            conn = Connection(certificate_id=c.id)
            conn.ip_client = ip_client
            conn.save()
        else:
            sys.exit(1)
