# -*- coding: utf-8 -*-
from django.template.loader import render_to_string

import os, os.path
from datetime import *
from OpenSSL import crypto
from jmb.openvpn.certgen import *

from django.db import models
from django.conf import settings
from django.contrib.auth.models import User
from django.utils.translation import ugettext_lazy as _

#from jmb.core.shortcuts import render_to_string
#from jmb.core.utils.functions import get_settings
from jmb.core.db import models as jumbo_models

from jmb.openvpn.utils import create_tar_gz

###########################
# KEY                     #
###########################
class Key(jumbo_models.DateModel, jumbo_models.UserModel, jumbo_models.StatusModel):
    ## FOREIGNKEY
    user = models.ForeignKey(
        User,
        null = True, blank = True,
        verbose_name = _("User"),
    )
    identity = models.CharField(
        null = True, blank = True,
        verbose_name = _("identity"),
        max_length = 20
    )

    ## CAMPI MODELLO
    path_pub = models.FilePathField(
        path="/etc/openvpn/keys",
        verbose_name = _("Path public"),
    )
    path_priv = models.FilePathField(
        path="/etc/openvpn/keys",
        verbose_name = _("Path private"),
    )

    def __unicode__(self):
        return self.path

################################
# CERTIFICATE                  #
################################
class Certificate(jumbo_models.DateModel, jumbo_models.UserModel, jumbo_models.StatusModel):
    ## FOREIGNKEY
    user = models.ForeignKey(
        User,
        null = True, blank = True,
        verbose_name = _("User"),
    )
    group = models.ForeignKey(
        "FwGroup",
        null = True, blank = True,
        verbose_name = _("group"),
    )
    ## CAMPI MODELLO
    start_date = models.DateField(
        null = True, blank = True,
        verbose_name = _("From"),
        help_text = _("""Se non si inserisce la start_date, il certificato
                       avra' come data di partenza di validita' la data di ieri.
                       Non inserendo nessuna delle due date il certificato avra'
                       validita' di 1 anno a partire da ieri.""")
    )
    stop_date =  models.DateField(
        null = True, blank = True,
        verbose_name = _("To"),
        help_text = _("""Se non si inserisce la stop_date, il certificato
                        avra' come data di fine validita' la data di oggi piu' 365 giorni.
                        Non inserendo nessuna delle due date il certificato avra'
                        validita' di 1 anno a partire da ieri.""")
    )
    description = models.CharField(
        max_length = 255,
        verbose_name = _("Description"),
        help_text = _("Si suggerisce di mettere il nome della persona a cui verra' dato il certificato")
    )
    email = models.EmailField(
        null = True, blank = True,
        max_length = 30,
        verbose_name = _("email"),
    )
    cn = models.CharField(
        null = True, blank = True,
        max_length = 100,
        verbose_name = _("Common Name"),
    )
    # Richiesta di certificato - File temporaneo
    csr = models.FilePathField(
        null = True, blank = True,
        path="/etc/openvpn/keys/crt",
        verbose_name = _("Certificate Request"),
    )

    def save(self, *args, **kws):
        self.cn = self.user.username
        if not self.start_date:
            self.start_date = date.today() - timedelta(days=1)
        if not self.stop_date:
            self.stop_date = date.today() + timedelta(days=365)
        super(Certificate, self).save(*args, **kws)
        create_tar_gz(self)
        self.refresh_rules()

    def delete(self, *args, **kws):
        super(Certificate, self).delete(*args, **kws)
        self.delete_certificate_file()

    def refresh_rules(self):

        fw_file = os.path.join(settings.OVPN_CERT_RULES_PATH, "%s.fw" % self.cn)

        if not self.group:
            return

        in_rules, fw_rules = self.group.get_rules(self.cn)
        conf_file = render_to_string("openvpn/default_cert.fw", {'cn':self.cn, 'fw_rules':fw_rules, 'in_rules':in_rules})
        f = open(fw_file, 'w')
        f.write(conf_file)
        f.write('\n')
        f.close()

    def get_file_certificate(self):
        TAR_PATH = settings.OVPN_TAR_PATH
        f_path = os.path.join(TAR_PATH, self.cn)

    def delete_connections(self):
        Connection.objects.filter(certificate=self).delete()

    def delete_certificate_file(self):
        EXT_CERT = settings.OVPN_EXT_CERT
        TAR_PATH = settings.OVPN_TAR_PATH
        CERT_RULES_PATH = settings.OVPN_CERT_RULES_PATH
        fname = "%s.%s" % (self.cn, EXT_CERT)

        # Cancello il file contenente il certificato
        f_path = os.path.join(TAR_PATH, fname)
        try:
            os.remove(f_path)
        except:
            pass

        # Cancello il file contenente le regole del firewall
        fname = "%s.fw" % (self.cn, )
        f_path = os.path.join(CERT_RULES_PATH, fname)
        try:
            os.remove(f_path)
        except:
            pass

    def __unicode__(self):
        return self.description

    class Meta:
        verbose_name = _("Certificate")
        verbose_name_plural = _("Certificates")
        permissions = (
            ('can_view_certificate', 'Can View Certificate'),
        )

###############################
# CONNECTION                  #
###############################
class Connection(jumbo_models.DateModel):
    ## FOREIGNKEY
    certificate = models.ForeignKey(
        Certificate,
        verbose_name = _("Certificate"),
    )
    start_date = models.DateTimeField(
        auto_now_add = True,
        verbose_name = _("Start")
    )
    stop_date = models.DateTimeField(
        blank = True, null = True,
        verbose_name = _("Stop")
    )
    ip_client = models.CharField(
        null = True, blank = True,
        max_length = 20,
        verbose_name = _("Client IP"),
    )

    def __unicode__(self):
        return unicode(self.certificate)

    class Meta:
        verbose_name = _("Connection")
        verbose_name_plural = _("Connections")
        permissions = (
            ('can_view_connection', 'Can View Connection'),
            ('manage_openvpn', 'Can Manage OpenVPN'),
            ('get_status_openvpn', 'Can View OpenVPN Status'),
            ('onoff_openvpn', 'Can ON/OFF OpenVPN'),
            ('restart_openvpn', 'Can Restart OpenVPN'),
        )

###########################
# FW RULE                 #
###########################
PROTOCOL = (
    ('TCP', 'TCP'),
    ('UDP', 'UDP'),
    ('ICM', 'ICMP')
)

ACTION = (
    ('AC', 'ACCEPT'),
    ('DR', 'DROP')
)

TABLE = (
    ('F', 'FILTER'),
)

CHAIN = (
    ('IN', 'INPUT'),
    ('FW', 'FORWARD')
)

class FwRule(jumbo_models.DateModel, jumbo_models.UserModel, jumbo_models.StatusModel):
    ## FOREIGNKEY
    groups = models.ManyToManyField(
        "FwGroup",
        related_name = "rules"
    )
    ## CAMPI MODELLO
    ip_src = models.CharField(
        null = True, blank = True,
        max_length = 20,
        verbose_name = _("Source IP"),
    )
    ip_dest = models.CharField(
        null = True, blank = True,
        max_length = 20,
        verbose_name = _("Destination IP"),
    )
    port_dest = models.IntegerField(
        null = True, blank = True,
        verbose_name = _("Destination Port")
    )
    proto = models.CharField (
        null = True, blank = True,
        max_length = 3,
        verbose_name = _("Protocol"),
        choices = PROTOCOL
    )
    rules = models.CharField (
        null = True, blank = True,
        verbose_name = _("Rules"),
        max_length = 255,
    )
    action = models.CharField (
        verbose_name = _("Action"),
        max_length = 2,
        choices = ACTION
    )
    table = models.CharField (
        max_length = 1,
        verbose_name = _("Table"),
        choices = TABLE
    )
    chain = models.CharField (
        max_length = 2,
        verbose_name = _("Chain"),
        choices = CHAIN
    )
    order = models.IntegerField(
        verbose_name = _("Order")
    )

    def save(self, *args, **kws):
        ## FIX ME
        super(FwRule, self).save(*args, **kws)

    def delete(self):
        gruppi = [ g for g in self.group.all() ]
        super(FwRule, self).delete()

        for g in gruppi:
            l_cert = Certificate.objects.filter(group__exact=g.id)
            for c in l_cert:
                c.refresh_rules()

    def get_groups(self):
        return ','.join([g.name for g in self.groups.all() ])
    get_groups.short_description = _("Groups")

    def getRuleString(self, cn):
        s = ""
        if self.chain == 'IN':
            s = "iptb -A tuni_%s" % cn

        if self.chain == 'FW':
            s = "iptb -A tunf_%s" % cn

        if self.ip_dest:
            s = s + " -d %s" % self.ip_dest

        if self.proto:
            s = s + " -p %s" % self.get_proto_display()

        if self.port_dest:
            s = s + " --dport %s" % self.port_dest

        s = s + " -j %s" % self.get_action_display()
        return s

    def __unicode__(self):
        return u"%s %s %s" % (self.get_table_display(), self.get_chain_display(), self.get_action_display())

    class Meta:
        verbose_name = _("Rule")
        verbose_name_plural = _("Rules")
        permissions = (
            ('can_view_fwrule', 'Can View Rules'),
        )

############################
# FW GROUP                 #
############################
class FwGroup(jumbo_models.DateModel, jumbo_models.UserModel, jumbo_models.StatusModel):
    name = models.CharField (
        max_length = 100,
        verbose_name = _("Name"),
        unique=True
    )
    # commentare quando si fa il syncdb iniziale
    #rule = models.ManyToManyField(
    #    FwRule,
    #    blank = True, null = True,
    #    db_table = 'openvpn_fwrule_group'
    #)

    def get_rules(self, cn):
        groups = FwGroup.objects.get(id=self.id)

        fw_rules = ""
        in_rules = ""

        for rule in groups.rules.order_by('chain','order'):
            r = rule.getRuleString(cn)
            if rule.chain == 'IN':
                in_rules += "%s\n" % r

            if rule.chain == 'FW':
                fw_rules += "%s\n" % r

        return [in_rules, fw_rules]

    def __unicode__(self):
        return u"%s" % (self.name)

    class Meta:
        verbose_name = _("Group")
        verbose_name_plural = _("Groups")
        permissions = (
            ('can_view_fwgroup', 'Can View Group'),
        )

############################
# FW GROUP RULES           #
############################
#class FwGroupsRules(models.Model):
#    rule = models.ForeignKey(FwRule)
#    group = models.ForeignKey(FwGroup)

#    def __unicode__(self):
#        return u'%s - %s' % (self.rule, self.group)
