# -*- coding: utf-8 -*-

import zipfile
import tarfile, tempfile
import os
from datetime import date

from OpenSSL import crypto
from django.template.loader import render_to_string
from django.conf import settings

from jmb.openvpn import certgen

def create_zip(tar_name, key_name, cert_name, conf_user_file, ca_cert):
    z = zipfile.ZipFile("%s.zip" % tar_name, "w")
    z.write(key_name, '/openvpn/%s' % os.path.basename(key_name))
    z.write(cert_name, '/openvpn/%s' % os.path.basename(cert_name))
    z.write(conf_user_file, '/openvpn/%s' % os.path.basename(conf_user_file))
    z.write(ca_cert, '/openvpn/ca.crt')
    z.close()

def create_tar(tar_name, key_name, cert_name, conf_user_file, ca_cert):
    tar = tarfile.open("%s.tar.gz" % tar_name, "w:gz")

    for path, name in ((key_name, key_name), (cert_name, cert_name),
                       (conf_user_file, conf_user_file), (ca_cert, "ca.crt")):
        tarinfo = tar.gettarinfo(path, 'openvpn/'+ name)
        tar.addfile(tarinfo, file(path))
    tar.close()

def buf(filename):
    f = open(filename)
    buf = f.read()
    f.close()
    return buf

def create_tar_gz(certificate):

    id = certificate.id

    KEY_NAME       = os.path.join(settings.OVPN_TAR_PATH, '%s.key' % certificate.cn)
    CERT_NAME      = os.path.join(settings.OVPN_TAR_PATH, "%s.crt" % certificate.cn)
    CONF_USER_FILE = os.path.join(settings.OVPN_TAR_PATH, "%s.ovpn" % certificate.cn)
    TAR_NAME       = os.path.join(settings.OVPN_TAR_PATH, "%s" % certificate.user.username)

    # CA private key and certificate
    cakey = crypto.load_privatekey(crypto.FILETYPE_PEM, buf(settings.OVPN_CA_KEY))
    cacert = crypto.load_certificate(crypto.FILETYPE_PEM, buf(settings.OVPN_CA_CERT))

    # Creazione coppia di chiavi
    user_prkey = certgen.createKeyPair(certgen.TYPE_RSA, 1024)
    str_user_prkey = crypto.dump_privatekey(crypto.FILETYPE_PEM, user_prkey)

    # Creazione file chiavi pubbliche e private
    f_pkey = open(KEY_NAME, 'w')
    f_pkey.write(str_user_prkey)
    f_pkey.write('\n')
    f_pkey.flush()

    # creazione certificato
    req = certgen.createCertRequest(user_prkey, CN=certificate.cn)

    start_delta = certificate.start_date or date.today() - timedelta(days=1)
    start_delta = start_delta - date.today()
    notBefore = start_delta.days * 86400

    stop_delta = certificate.stop_date or date.today() + timedelta(days=365)
    stop_delta = stop_delta - date.today()
    notAfter = stop_delta.days * 86400

    seconds = (notBefore, notAfter)
    cert = certgen.createCertificate(req, (cacert, cakey), 1000, seconds)
    f_crt = open(CERT_NAME, 'w')
    f_crt.write(crypto.dump_certificate(crypto.FILETYPE_PEM, cert))
    f_crt.write('\n')
    f_crt.flush()

    # Creazione file di configurazione .opvn
    c = open(OVPN_CONF_DEFAULT_FILE)
    remote = ''
    for line in c.readlines():
        if line.startswith('REMOTE='):
            remote = line[line.index('=')+1:]

    assert remote, "Missing REMOTE var in conf file %s" % OVPN_CONF_DEFAULT_FILE

    conf_file = render_to_string("openvpn/client_conf.ovpn", {
        'remote':remote,
        'udp_proto' : settings.OVPN_PROTO == 'UDP', 
        'cert': os.path.basename(CERT_NAME),
        'key': os.path.basename(KEY_NAME),
        'comp_lzo' : settings.OVPN_COMP_LZO,
        'use_password': settings.OVPN_USE_PASSWORD})

    #f_conf = tempfile.NamedTemporaryFile(prefix='%s' % user,suffix = '.%s' % DATI_CONF['EXT'], mode='w+b')
    f_conf = open(CONF_USER_FILE, 'w')
    f_conf.write(conf_file)
    f_conf.write('\n')
    f_conf.flush()

    if settings.OVPN_EXT_CERT == 'zip':
        create_zip(TAR_NAME, KEY_NAME, CERT_NAME, CONF_USER_FILE, settings.OVPN_CA_CERT)
    else:
        create_tar(TAR_NAME, KEY_NAME, CERT_NAME, CONF_USER_FILE, settings.OVPN_CA_CERT)

    os.remove(KEY_NAME)
    os.remove(CERT_NAME)
    os.remove(CONF_USER_FILE)
